import { Component } from "react";

const targetStack = [];
function pushTarget(_target) {
    if (_target) targetStack.push(_target);
    Fix.Model.target = _target;
}

function popTarget() {
    Fix.Model.target = targetStack.pop();
}

export function inject(storeFn) {
    return function (ComponentClass) {
        return class extends Component {
            constructor(props, context) {
                super(props, context);
                this.store = storeFn && new storeFn(props);
                var flag = false;
                this.watcher = new Fix.Watcher(
                    this,
                    () => {
                        flag &&
                        this.forceUpdate();
                    },
                    null
                );
                flag = true;
            }
            static displayName = "Inject" +
                (ComponentClass.displayName || ComponentClass.name);
            shouldComponentUpdate() {
                return false;
            }
            componentWillMount() {
                pushTarget(this.store);
                Fix.pushTarget(this.watcher);
            }
            render() {
                return (
                    <ComponentClass
                        {...this.props}
                        store={this.store}
                        model={this.store.model}
                    />
                );
            }
            componentDidMount() {
                Fix.popTarget();
                popTarget();
            }
            componentWillUpdate() {
                pushTarget(this.store);
                Fix.pushTarget(this.watcher);
            }
            componentDidUpdate() {
                Fix.popTarget();
                popTarget();
            }
            componentWillUnmount() {
                this.watcher.teardown();
                this.store.destroy();
            }
        };
    };
}

export default inject;
